<?php
/**
 * "Get Resolver Links" AJAX handler
 *
 * PHP version 7
 *
 * Copyright (C) Villanova University 2018.
 * Copyright (C) Leipzig University Library 2019.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  AJAX
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Graham Seaman <Graham.Seaman@rhul.ac.uk>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Frank Morgner <morgnerf@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\AjaxHandler;

use VuFind\Resolver\Connection;
use VuFind\Resolver\Driver\PluginManager as ResolverManager;
use VuFind\Session\Settings as SessionSettings;
use Zend\Config\Config;
use Zend\Mvc\Controller\Plugin\Params;
use Zend\View\Renderer\RendererInterface;

/**
 * "Get Resolver Links" AJAX handler
 *
 * Fetch Links from resolver given an OpenURL and format as HTML
 * and output the HTML content in JSON object.
 *
 * @category VuFind
 * @package  AJAX
 * @author   Demian Katz <demian.katz@villanova.edu>
 * @author   Graham Seaman <Graham.Seaman@rhul.ac.uk>
 * @author   Gregor Gawol <gawol@ub.uni-leipzig.de>
 * @author   Frank Morgner <morgnerf@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development Wiki
 */
class GetResolverLinks extends \VuFind\AjaxHandler\GetResolverLinks
{
    /**
     * Resolver configuration
     *
     * @var Config
     */
    protected $resolverConfig;

    /**
     * Constructor
     *
     * @param SessionSettings   $ss       Session settings
     * @param ResolverManager   $pm       Resolver driver plugin manager
     * @param RendererInterface $renderer View renderer
     * @param Config            $config   Top-level VuFind configuration (config.ini)
     * @param Config            $resolver Link Resolver configuration (Resolver.ini)
     */
    public function __construct(
        SessionSettings $ss,
        ResolverManager $pm,
        RendererInterface $renderer,
        Config $config,
        Config $resolver
    ) {
        parent::__construct($ss, $pm, $renderer, $config);
        $this->resolverConfig = $resolver;
    }

    /**
     * Handle a request.
     *
     * Method can handle multiple link resolvers. VuFind standard configuration
     * of [OpenUrl] config.ini is replaced by finc defined Resolver.ini. Therefore
     * parameter $config is currently out of use.
     *
     * @param Params $params Parameter helper from controller
     *
     * @return array [response data, HTTP status code]
     */
    public function handleRequest(Params $params)
    {
        $this->disableSessionWrites();  // avoid session write timing bug
        $openUrl = $params()->fromQuery('openurl', '');
        $resolver = $params()->fromQuery('resolvertype', '');
        $searchClassId = $params()->fromQuery('searchClassId', '');

        //$config = $this->getConfig('Resolver');
        $resolvers = explode(',', $this->resolverConfig->General->active_resolvers);

        if (in_array($resolver, $resolvers)
            && isset($this->resolverConfig->$resolver)
        ) {
            $resolverType = $this->resolverConfig->$resolver->resolver ?? 'generic';
            if (!$this->pluginManager->has($resolverType)) {
                return $this->formatResponse(
                    $this->translate("Could not load driver for $resolverType"),
                    self::STATUS_HTTP_ERROR
                );
            }

            /**
             * Loaded Resolver
             *
             * @var DriverInterface $resolverObject
             */
            $resolverObject
                = new Connection($this->pluginManager->get($resolverType));
            if (isset($this->resolverConfig->$resolver->resolver_cache)) {
                $resolverObject->enableCache(
                    $this->resolverConfig->$resolver->resolver_cache
                );
            }
            $result = $resolverObject->fetchLinks($openUrl);

            // Sort the returned links into categories based on service type:
            $electronic = $print = $services = $unknown = [];
            foreach ($result as $link) {
                switch ($link['service_type'] ?? '') {
                case 'getHolding':
                    $print[] = $link;
                    break;
                case 'getWebService':
                    $services[] = $link;
                    break;
                case 'getDOI':
                    // Special case -- modify DOI text for special display:
                    $link['title'] = $this->translate('Get full text');
                    $link['coverage'] = '';
                        break;
                    case 'unknown':
                        $unknown[] = $link;
                    break;
                case 'getFullTxt':
                default:
                    $electronic[] = $link;
                    break;
                }
            }

            // Get the OpenURL base:
            if (isset($this->resolverConfig->$resolver)
                && isset($this->resolverConfig->$resolver->url)
            ) {
                // Trim off any parameters (for legacy compatibility
                // -- default config used to include extraneous parameters):
                [$base] = explode(
                    '?',
                    $this->resolverConfig->$resolver->url
                );
            } else {
                $base = false;
            }

            $moreOptionsLink = $resolverObject->supportsMoreOptionsLink()
                ? $resolverObject->getResolverUrl($openUrl) : '';

            // Render the links using the view:
            $view = [
                'openUrlBase' => $base,
                'openUrl' => $openUrl,
                'electronic' => $electronic,
                'unknown' => $unknown,
                'services' => $services,
                'searchClassId' => $searchClassId,
                'resolver' => $resolver,
                'moreOptionsLink' => $moreOptionsLink
            ];
            $html = $this->renderer->render('ajax/resolverLinks.phtml', $view);

            // Render GetItBox specific templates
            $viewPrint = [
                'print' => $print,
                'messages' => (isset($result['messages']) ? array_unique($result['messages']) : [])
            ];
            $print = $this->renderer->render('ajax/resolverLinks-print.phtml', $viewPrint);

            $viewButton = [
                'showOrderButton' => ($result['access'] ?? []),
                'recordId' => $params->fromQuery('recordId', '')
            ];
            $button = $this->renderer->render(
                'ajax/resolverLinks-button.phtml',
                $viewButton
            );

            // output HTML encoded in JSON object
            return $this->formatResponse(compact('html', 'print', 'button'));
        }

        // if we get here, the requested resolver is not configured
        return $this->formatResponse(
            $this->translate("Resolver $resolver not configured"),
            self::STATUS_HTTP_ERROR
        );
    }
}
