<?php
/**
 * Order hydrator
 *
 * Copyright (C) 2021 Leipzig University Library
 *
 * PHP Version 7
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * @category VuFind
 * @package  Hydrator
 * @author   Alexander Purr <purr@ub.uni-leipzig.de>
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @author   Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
namespace fid\Hydrator;

use fid\Service\DataTransferObject\Order;
use VuFind\RecordDriver\DefaultRecord;
use Zend\Hydrator\AbstractHydrator;

/**
 * Order hydrator for saving aquistions data correctly to dto before sending to fidis
 *
 * @category VuFind
 * @package  Hydrator
 * @author   Alexander Purr <purr@ub.uni-leipzig.de>
 * @author   Robert Lange <lange@ub.uni-leipzig.de>
 * @author   Sebastian Kehr <kehr@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU GPLv2
 * @link     https://vufind.org/wiki/development Wiki
 */
class OrderHydrator extends AbstractHydrator
{
    /**
     * Extract method:
     * Not implemented, not in use
     *
     * @param Order $object Order object
     *
     * @return void
     */
    public function extract($object)
    {
    }

    /**
     * Save acquisition data to order object
     *
     * @param array $data   Relevant data
     * @param Order $object Order object
     *
     * @return Order|void
     */
    public function hydrate(array $data, $object)
    {
        /* @var DefaultRecord $driver */
        $driver = $data['driver'];

        $object->setType($data['type']);
        $object->setUser($data['user']);

        $partialCopy = array_key_exists('subitoPartialCopy', $data)
            ? $data['subitoPartialCopy'] : null;
        $digitization = array_key_exists('digitization', $data)
            ? $data['digitization'] : null;
        $pda = array_key_exists('pda', $data) ? $data['pda'] : null;

        /* Collect needed record data */
        $id = $driver->tryMethod('getUniqueID');
        $title = $driver->tryMethod('getTitle');
        $pdetails = $driver->tryMethod('getPublicationDetails');
        $publicationDetails
            = $this->toArray(!empty($pdetails) ? $pdetails[0] : null);
        $authors = $driver->tryMethod('getDeduplicatedAuthors');
        $primaryAuthor = $driver->tryMethod('getPrimaryAuthor');
        $secondaryAuthors = $driver->tryMethod('getSecondaryAuthors');
        $isbn = $driver->tryMethod('getCleanISBN');
        $issn = $driver->tryMethod('getCleanISSN');
        $languages = $driver->tryMethod('getLanguages');
        $price = $driver->tryMethod('getPrice');
        /* pre generated record-url */
        $url = $data['url'];
        /* Collect special journal data used for subito-article*/
        $publishedIn = $driver->tryMethod('getPublishedIn');
        if (isset($publishedIn['issue'])) {
            $publishedIn['issue'] = is_string($publishedIn['issue'])
                ? $publishedIn['issue'] : implode(', ', $publishedIn['issue']);
        }

        if (empty($publishedIn['issns'])) {
            $publishedIn['issns'] = [""];
        }

        $record = compact(
            'id',
            'title',
            'publicationDetails',
            'authors',
            'primaryAuthor',
            'secondaryAuthors',
            'issn',
            'isbn',
            'languages',
            'price',
            'publishedIn',
            'url'
        );

        $object->setData(compact('record', 'partialCopy', 'digitization', 'pda'));
        $object->setStatus($data['status']);
        $object->setLabel($data['label']);
    }

    /**
     * Helper function to convert object member variables to array.
     * Member name becomes array key.
     *
     * @param $object Object
     *
     * @return array
     */
    protected function toArray($object) : array
    {
        $array = (array)$object;
        $keys = str_replace('*', '', array_keys($array));
        return array_combine($keys, array_values($array));
    }
}
