<?php
/**
 * EZB Link Resolver Driver
 *
 * EZB is a free service -- the API endpoint is available at
 * http://services.dnb.de/fize-service/gvr/full.xml
 *
 * API documentation is available at
 * http://www.zeitschriftendatenbank.de/services/journals-online-print
 *
 * PHP version 7
 *
 * Copyright (C) Markus Fischer, info@flyingfischer.ch
 *
 * last update: 2011-04-13
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2,
 * as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * @category VuFind
 * @package  Resolver_Drivers
 * @author   Markus Fischer <info@flyingfischer.ch>
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:link_resolver_drivers Wiki
 */
namespace finc\Resolver\Driver;

use DOMDocument;
use DOMXpath;
use VuFind\I18n\Translator\TranslatorAwareInterface as TranslatorAwareInterface;
use VuFind\Resolver\Driver\Ezb as Base;

/**
 * EZB Link Resolver Driver
 *
 * @category VuFind
 * @package  Resolver_Drivers
 * @author   Markus Fischer <info@flyingfischer.ch>
 * @author   André Lahmann <lahmann@ub.uni-leipzig.de>
 * @license  http://opensource.org/licenses/gpl-2.0.php GNU General Public License
 * @link     https://vufind.org/wiki/development:plugins:link_resolver_drivers Wiki
 */
class Ezb extends Base implements TranslatorAwareInterface
{
    use \VuFind\I18n\Translator\TranslatorAwareTrait;
    use \finc\Resolver\Driver\EzbTrait;

    /**
     * Base URL for link resolver
     *
     * @var string
     */
    protected $baseUrl;

    /**
     * Resolver configuration
     *
     * @var \Zend\Config\Resolver $resolver
     */
    protected $config;

    /**
     * HTTP client
     *
     * @var \Zend\Http\Client
     */
    protected $httpClient;

    /**
     * Constructor
     *
     * @param string            $config     ezb configuration
     * @param \Zend\Http\Client $httpClient HTTP client
     */
    public function __construct($config, \Zend\Http\Client $httpClient)
    {
        $this->baseUrl = $config->url;
        $this->httpClient = $httpClient;
        $this->config = $config;
    }

    /**
     * Fetch Links
     *
     * Fetches a set of links corresponding to an OpenURL
     *
     * @param string $openURL openURL (url-encoded)
     *
     * @return string         raw XML returned by resolver
     */
    public function fetchLinks($openURL)
    {
        $url = $this->getResolverUrl($openURL);

        // Make the call to the EZB and load results
        $feed = $this->httpClient->setUri($url)->send()->getBody();
        return $feed;
    }

    /**
     * Parse Links
     *
     * Parses an XML file returned by a link resolver
     * and converts it to a standardised format for display
     *
     * @param string $xmlstr Raw XML returned by resolver
     *
     * @return array         Array of values
     */
    public function parseLinks($xmlstr)
    {
        $records = []; // array to return

        $xml = new DOMDocument();
        if (!@$xml->loadXML($xmlstr)) {
            return $records;
        }

        $xpath = new DOMXpath($xml);

        // get results for online
        $this->getElectronicResults('0', 'Free', $records, $xpath);
        $this->getElectronicResults('1', 'Partially free', $records, $xpath);
        $this->getElectronicResults('2', 'Licensed', $records, $xpath);
        $this->getElectronicResults('3', 'Partially licensed', $records, $xpath);
        $this->getElectronicResults('4', 'Not free', $records, $xpath);
        $this->getElectronicResults('10', 'Unknown Electronic', $records, $xpath);

        // get results for print, only if available
        $this->getPrintResults('2', 'Print available', $records, $xpath);
        $this->getPrintResults('3', 'Print partially available', $records, $xpath);
        $this->getPrintResults('10', 'Unknown Print', $records, $xpath);

        return $records;
    }

    /**
     * Get Resolver Url
     *
     * Transform the OpenURL as needed to get a working link to the resolver.
     *
     * @param string $openURL openURL (url-encoded)
     *
     * @return string Link
     */
    public function getResolverUrl($openURL)
    {
        // Unfortunately the EZB-API only allows OpenURL V0.1 and
        // breaks when sending a non expected parameter (like an ISBN).
        // So we do have to 'downgrade' the OpenURL-String from V1.0 to V0.1
        // and exclude all parameters that are not compliant with the EZB.

        // Parse OpenURL into associative array:
        $tmp = explode('&', $openURL);
        $parsed = [];

        foreach ($tmp as $current) {
            $tmp2 = explode('=', $current, 2);
            $parsed[$tmp2[0]] = $tmp2[1];
        }

        // Downgrade 1.0 to 0.1
        if ($parsed['ctx_ver'] == 'Z39.88-2004') {
            $openURL = $this->downgradeOpenUrl($parsed);
        }

        // BOF - finc specific PID-Zone parameter

        // use JOP supported identifier bibid (finc specific as VuFind does not
        // support resolver specific configuration)
        if (isset($this->config->bibid)) {
            $pid = 'bibid=' . $this->config->bibid;
        } else {
            // use IP-based request as fallback
            $pid = 'client_ip=' . $_SERVER['REMOTE_ADDR'];
        }

        // use finc indexed zdbid if issn is not given
        $pid .= !isset($parsed['rft.issn']) && isset($parsed['zdbid']) ?
            '&zdbid=' . $parsed['zdbid'] : '';

        // limit results to ezb only (set to true)
        if (isset($this->config->ezb_only) && $this->config->ezb_only) {
            $pid .= '&ezb=1';
        }
        // limit results to zdb only (set to true)
        if (isset($this->config->zdb_only) && $this->config->zdb_only) {
            $pid .= '&zdb=1';
        }

        $openURL .= '&pid=' . urlencode($pid);

        // EOF - finc specific PID-Zone parameter

        // as of October 2020 the JOP resolver supports DOIs and we want to use
        // DOIs for request in finc
        $openURL .= isset($parsed['doi']) ? '&id=doi:' . $parsed['doi'] : '';

        // Make the call to the EZB and load results
        $url = $this->baseUrl . '?' . $openURL;

        return $url;
    }

    /**
     * Allows for resolver driver specific enabling/disabling of the more options
     * link which will link directly to the resolver URL. This should return false if
     * the resolver returns data in XML or any other human unfriendly response.
     *
     * @return bool
     */
    public function supportsMoreOptionsLink()
    {
        // the EZB link resolver returns unstyled XML which is not helpful for the
        // user
        return false;
    }

    /**
     * Extract electronic results from the EZB response and inject them into the
     * $records array.
     *
     * @param string   $state    The state attribute value to extract
     * @param string   $coverage The coverage string to associate with the state
     * @param array    $records  The array of results to update
     * @param DOMXpath $xpath    The XPath object containing parsed XML
     *
     * @return void
     */
    protected function getElectronicResults($state, $coverage, &$records, $xpath)
    {
        $results = $xpath->query(
            "/OpenURLResponseXML/Full/ElectronicData/ResultList/Result[@state=" .
            $state . "]"
        );

        /*
         * possible state values:
         * -1 ISSN nicht eindeutig
         *  0 Standort-unabhängig frei zugänglich
         *  1 Standort-unabhängig teilweise zugänglich (Unschärfe bedingt durch
         *    unspezifische Anfrage oder Moving-Wall)
         *  2 Lizenziert
         *  3 Für gegebene Bibliothek teilweise lizenziert (Unschärfe bedingt durch
         *    unspezifische Anfrage oder Moving-Wall)
         *  4 nicht lizenziert
         *  5 Zeitschrift gefunden
         *    Angaben über Erscheinungsjahr, Datum ... liegen außerhalb des
         *    hinterlegten bibliothekarischen Zeitraums
         * 10 Unbekannt (ISSN unbekannt, Bibliothek unbekannt)
         */
        $state_access_mapping = [
            '-1' => 'error',
            '0'  => 'open',
            '1'  => 'open',
            '2'  => 'limited',
            '3'  => 'limited',
            '4'  => 'denied',
            '5'  => 'denied',
            '10' => 'unknown'
        ];

        $i = 0;
        foreach ($results as $result) {
            $record = [];

            // get title from XPath Element defined in $xpathTitleSelector
            $titleXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/" .
                "Result[@state={$state}][" . ($i + 1) . "]/" .
                static::$xpathTitleSelector;
            $title = $xpath->query($titleXP, $result)->item(0);
            if (isset($title)) {
                $record['title'] = strip_tags($title->nodeValue);
            }

            // get additional coverage information
            $additionalXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/" .
                "Result[@state={$state}][".($i+1)."]/Additionals/Additional";
            $additionalType = ['nali', 'intervall', 'moving_wall'];
            $additionals = [];
            foreach ($additionalType as $type) {
                $additional = $xpath
                    ->query($additionalXP . "[@type='".$type."']", $result)->item(0);
                if (isset($additional->nodeValue)) {
                    $additionals[$type] = strip_tags($additional->nodeValue);
                }
            }
            $record['coverage']
                = !empty($additionals) ? implode(";", $additionals) : $coverage;

            $record['access'] = $state_access_mapping[$state];

            // try to find direct access URL
            $accessUrlXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/" .
                "Result[@state={$state}][".($i+1)."]/AccessURL";
            $accessUrl = $xpath->query($accessUrlXP, $result)->item(0);

            // try to find journal URL as fallback for direct access URL
            $journalUrlXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/" .
                "Result[@state={$state}][" . ($i + 1) . "]/JournalURL";
            $journalUrl = $xpath->query($journalUrlXP, $result)->item(0);

            // return direct access URL if available
            // otherwise journal URL as fallback
            if (isset($accessUrl->nodeValue)) {
                $record['href'] = $accessUrl->nodeValue;
            } elseif (isset($journalUrl)) {
                $record['href'] = $journalUrl->nodeValue;
            }

            $readmeXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/" .
                "Result[@state={$state}][".($i+1)."]/ReadmeURL[@lang='" .
                $this->getTranslatorLocale()."']";
            $readme = $xpath->query($readmeXP, $result)->item(0);
            if (isset($readme->nodeValue)) {
                $record['coverageHref'] = $readme->nodeValue;
            }

            // Service type needs to be hard-coded for calling code to properly
            // categorize links. The commented code below picks a more appropriate
            // value but won't work for now -- retained for future reference.
            //$service_typeXP = "/OpenURLResponseXML/Full/ElectronicData/ResultList/"
            //    . "Result[@state={$state}][".($i+1)."]/AccessLevel";
            //$record['service_type']
            //    = $xpath->query($service_typeXP, $result)->item(0)->nodeValue;
            $record['service_type'] = 'getFullTxt';
            array_push($records, $record);
            $i++;
        }
    }

    /**
     * Extract print results from the EZB response and inject them into the
     * $records array.
     *
     * @param string   $state    The state attribute value to extract
     * @param string   $coverage The coverage string to associate with the state
     * @param array    $records  The array of results to update
     * @param DOMXpath $xpath    The XPath object containing parsed XML
     *
     * @return void
     */
    protected function getPrintResults($state, $coverage, &$records, $xpath)
    {
        $results = $xpath->query(
            "/OpenURLResponseXML/Full/PrintData/ResultList/Result[@state={$state}]"
        );

        /*
         * possible state values:
         * -1 ISSN nicht eindeutig
         *  2 Vorhanden
         *  3 Teilweise vorhanden (Unschärfe bedingt durch unspezifische Anfrage bei
         *    nicht vollständig vorhandener Zeitschrift)
         *  4 Nicht vorhanden
         * 10 Unbekannt (ZDB-ID unbekannt, ISSN unbekannt, Bibliothek unbekannt)
         */
        $state_access_mapping = [
            '-1' => 'error',
            '2'  => 'open',
            '3'  => 'limited',
            '4'  => 'denied',
            '10' => 'unknown'
        ];

        $i = 0;
        foreach ($results as $result) {
            $record = [];
            $record['title'] = $coverage;

            $resultXP = "/OpenURLResponseXML/Full/PrintData/ResultList/" .
                "Result[@state={$state}][" . ($i + 1) . "]";
            $resultElements = [
                'Title', 'Location', 'Signature', 'Period', 'Holding_comment'
            ];
            $elements = [];
            foreach ($resultElements as $element) {
                $elem = $xpath->query($resultXP . "/" . $element, $result)->item(0);
                if (isset($elem->nodeValue)) {
                    $elements[$element] = strip_tags($elem->nodeValue);
                }
            }
            $record['coverage']
                = !empty($elements) ? implode("; ", $elements) : $coverage;

            $record['access'] = $state_access_mapping[$state];

            $urlXP = "/OpenURLResponseXML/Full/PrintData/References/Reference/URL";
            $url = $xpath->query($urlXP, $result)->item($i);
            if (isset($url->nodeValue)) {
                $record['href'] = $url->nodeValue;
            }
            // Service type needs to be hard-coded for calling code to properly
            // categorize links. The commented code below picks a more appropriate
            // value but won't work for now -- retained for future reference.
            //$service_typeXP = "/OpenURLResponseXML/Full/PrintData/References"
            //    . "/Reference/Label";
            //$record['service_type']
            //    = $xpath->query($service_typeXP, $result)->item($i)->nodeValue;
            $record['service_type'] = 'getHolding';
            array_push($records, $record);
            $i++;
        }
    }
}
